#ifndef WS2812_STM32F4_H
#define WS2812_STM32F4_H

// helper macro for concatenation
#define CAT_(a,b)		a ## b
#define CAT(a,b)		CAT_(a,b)


// ----------------------------- definitions -----------------------------
#define WS2812_NR_LEDS			48

// warning: change source file if using tim1/8/9/10/11
// (different APB => change function calls/peripheral names/clock generation)
#define WS2812_TIMER_NR			3
#define WS2812_TIM_CHAN_NR		3
#define WS2812_OUT_PORT			B
#define WS2812_OUT_PIN			0

#define WS2812_TIM				CAT(TIM, 				WS2812_TIMER_NR)
#define WS2812_RCC_TIM			CAT(RCC_APB1Periph_TIM, WS2812_TIMER_NR)
#define WS2812_GPIO_AF_TIM 		CAT(GPIO_AF_TIM, 		WS2812_TIMER_NR)
#define WS2812_CCR				CAT(CCR, 				WS2812_TIM_CHAN_NR)
#define WS2812_SETCOMPARE		CAT(TIM_SetCompare, 	WS2812_TIM_CHAN_NR)
#define WS2812_DMA_SOURCE		CAT(TIM_DMA_CC, 		WS2812_TIM_CHAN_NR)
#define WS2812_TIM_CHANNEL		CAT(TIM_Channel_, 		WS2812_TIM_CHAN_NR)
#define WS2812_TIM_OC			CAT(TIM_OC, 			WS2812_TIM_CHAN_NR)
#define WS2812_TIM_OC_INIT		CAT(WS2812_TIM_OC, 		Init)
#define WS2812_TIM_OC_PRECFG	CAT(WS2812_TIM_OC, 		PreloadConfig)

#define WS2812_GPIO				CAT(GPIO, 				WS2812_OUT_PORT)
#define WS2812_RCC_GPIO			CAT(RCC_AHB1Periph_GPIO, WS2812_OUT_PORT)
#define WS2812_GPIO_PIN			CAT(GPIO_Pin_, 			WS2812_OUT_PIN)

#define WS2812_RCC_DMA			RCC_AHB1Periph_DMA1
#define WS2812_DMA_STREAM		DMA1_Stream7
#define WS2812_DMA_CHANNEL		DMA_Channel_5

// timer values to generate a "one" or a "zero" according to ws2812 datasheet
#define WS2812_PWM_ONE			15
#define WS2812_PWM_ZERO			7

// number of timer cycles (~1.25s) for the reset pulse
#define WS2812_RESET_LEN		50

// three colors per led, eight bits per color
#define WS2812_FRAMEBUF_LEN		(WS2812_NR_LEDS * 3 * 8 + WS2812_RESET_LEN)

#define WS2812_TIM_FREQ			21000000
#define WS2812_OUT_FREQ			800000



// ----------------------------- variables -----------------------------

extern volatile uint32_t ws2812_framebuffer[WS2812_NR_LEDS];



// ----------------------------- functions -----------------------------
void ws2812_init(void);
void ws2812_update_leds(void);
void ws2812_test_pattern(void);

#endif
